<?php
/**
 * The own template engine for loading caching and sending out images
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ImageTemplateEngine extends BaseTemplateEngine implements CompileableTemplate {
	/**
	 * Main nodes in the XML tree ("image" is ignored)
	 */
	private $mainNodes = array("type", "resolution", "background-color", "foreground-color", "image-string");

	/**
	 * Sub nodes in the XML tree
	 */
	private $subNodes = array("width", "height", "red", "green", "blue", "text");

	/**
	 * Image instance
	 */
	private $imageInstance = null;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set part description
		$this->setObjectDescription("Image template engine");

		// Create unique ID number
		$this->generateUniqueId();

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Creates an instance of the class TemplateEngine and prepares it for usage
	 *
	 * @param	$basePath		The local base path for all templates
	 * @param	$langInstance	An instance of LanguageSystem (default)
	 * @param	$ioInstance		An instance of FileIoHandler (default, middleware!)
	 * @return	$tplInstance	An instance of TemplateEngine
	 * @throws	BasePathIsEmptyException		If the provided $basePath is empty
	 * @throws	InvalidBasePathStringException	If $basePath is no string
	 * @throws	BasePathIsNoDirectoryException	If $basePath is no
	 *											directory or not found
	 * @throws	BasePathReadProtectedException	If $basePath is
	 *											read-protected
	 */
	public final static function createImageTemplateEngine ($basePath, ManageableLanguage  $langInstance, FileIoHandler $ioInstance) {
		// Get a new instance
		$tplInstance = new ImageTemplateEngine();

		// Is the base path valid?
		if (empty($basePath)) {
			// Base path is empty
			throw new BasePathIsEmptyException($tplInstance, self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
		} elseif (!is_string($basePath)) {
			// Is not a string
			throw new InvalidBasePathStringException(array($tplInstance, $basePath), self::EXCEPTION_INVALID_STRING);
		} elseif (!is_dir($basePath)) {
			// Is not a path
			throw new BasePathIsNoDirectoryException(array($tplInstance, $basePath), self::EXCEPTION_INVALID_PATH_NAME);
		} elseif (!is_readable($basePath)) {
			// Is not readable
			throw new BasePathReadProtectedException(array($tplInstance, $basePath), self::EXCEPTION_READ_PROTECED_PATH);
		}

		// Get configuration instance
		$cfgInstance = FrameworkConfiguration::getInstance();

		// Set the base path
		$tplInstance->setBasePath($basePath);

		// Initialize the variable stack
		$tplInstance->initVariableStack();

		// Set the language and IO instances
		$tplInstance->setLanguageInstance($langInstance);
		$tplInstance->setFileIoInstance($ioInstance);

		// Set template extensions
		$tplInstance->setRawTemplateExtension($cfgInstance->readConfig('raw_template_extension'));
		$tplInstance->setCodeTemplateExtension($cfgInstance->readConfig('code_template_extension'));

		// Absolute output path for compiled templates
		$tplInstance->setCompileOutputPath(PATH . $cfgInstance->readConfig('compile_output_path'));

		// Return the prepared instance
		return $tplInstance;
	}

	/**
	 * Renders the given image content
	 *
	 * @param	$imageContent		A valid XML image content
	 * @return	void
	 * @throws	XmlParserException	If an XML error was found
	 */
	public function renderImageContent ($imageContent) {
		// Get an XML parser
		$xmlParser = xml_parser_create();

		// Force case-folding to on
		xml_parser_set_option($xmlParser, XML_OPTION_CASE_FOLDING, true);

		// Set object
		xml_set_object($xmlParser, $this);

		// Set handler call-backs
		xml_set_element_handler($xmlParser, "startElement", "endElement");
		xml_set_character_data_handler($xmlParser, "characterHandler");

		// Now parse the XML tree
		if (!xml_parse($xmlParser, $imageContent)) {
			// Error found in XML!
			throw new XmlParserException(array($this, $xmlParser), BaseHelper::EXCEPTION_XML_PARSER_ERROR);
		} // END - if

		// Free the parser
		xml_parser_free($xmlParser);
	}

	/**
	 * Handles the start element of an XML resource
	 *
	 * @param	$resource		XML parser resource
	 * @param	$element		The element we shall handle
	 * @param	$attributes		All attributes
	 * @return	void
	 * @throws	InvalidXmlNodeException		If an unknown/invalid XML node name was found
	 */
	public function startElement ($resource, $element, array $attributes) {
		// Initial method name which will never be called...
		$methodName = "initImage";

		// Make the element name lower-case
		$element = strtolower($element);

		// Is the element a main node?
		if (in_array($element, $this->mainNodes)) {
			// Okay, main node found!
			$methodName = "setImage" . $this->convertToClassName($element);
		} elseif (in_array($element, $this->subNodes)) {
			// Sub node found
			$methodName = "setImageProperty" . $this->convertToClassName($element);
		} elseif ($element != "image") {
			// Invalid node name found
			throw new InvalidXmlNodeException(array($this, $element, $attributes), self::EXCEPTION_XML_NODE_UNKNOWN);
		}

		// Call method
		//* DEBUG: */ echo "call: ".$methodName."<br />\n";
		call_user_func_array(array($this, $methodName), $attributes);
	}

	/**
	 * Intializes the image
	 *
	 * @return	void
	 * @todo	Add cache creation here
	 */
	private function initImage () {
		// Unfinished work!
	}

	/**
	 * Set the image type
	 *
	 * @param	$imageType	Code fragment or direct value holding the image type
	 * @return	void
	 */
	private function setImageType ($imageType) {
		// Try to compile it first to get the value from variable stack
		$imageType = $this->compileRawCode($imageType);

		// Now make a class name of it
		$className = $this->convertToClassName($imageType."_image");

		// And try to initiate it
		$this->imageInstance = ObjectFactory::createObjectByName($className);
	}
}

// [EOF]
?>
