<?php
/**
 * A generic class for handling guests
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class Guest extends BaseFrameworkSystem implements ManageableGuest, Registerable {
	/**
	 * Instance of the database result
	 */
	private $resultInstance = null;

	/**
	 * Username of current user
	 */
	private $userName = "";

	/**
	 * Email of current user
	 */
	private $email = "";

	// Exceptions
	const EXCEPTION_USERNAME_NOT_FOUND   = 0x060;
	const EXCEPTION_USER_EMAIL_NOT_FOUND = 0x061;
	const EXCEPTION_USER_PASS_MISMATCH   = 0x062;

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className = "") {
		// Is the class name empty? Then this is not a specialized user class
		if (empty($className)) $className = __CLASS__;

		// Call parent constructor
		parent::__construct($className);

		// Set part description
		$this->setObjectDescription("Generic user class");

		// Create unique ID number
		$this->generateUniqueId();

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Creates an instance of this user class by a provided username. This
	 * factory method will check if the username is already taken and if not
	 * so it will throw an exception.
	 *
	 * @param	$userName		Username we need a class instance for
	 * @return	$userInstance	An instance of this user class
	 * @throws	UsernameMissingException	If the username does not exist
	 */
	public final static function createGuestByUsername ($userName) {
		// Get a new instance
		$userInstance = new Guest();

		// Set the username
		$userInstance->setUserName($userName);

		// Check if the username exists
		if (!$userInstance->ifUsernameExists()) {
			// Throw an exception here
			throw new UsernameMissingException(array($userInstance, $userName), self::EXCEPTION_USERNAME_NOT_FOUND);
		}

		// Return the instance
		return $userInstance;
	}

	/**
	 * Creates an instance of this user class by a provided email address. This
	 * factory method will not check if the email address is there.
	 *
	 * @param	$email			Email address of the user
	 * @return	$userInstance	An instance of this user class
	 */
	public final static function createGuestByEmail ($email) {
		// Get a new instance
		$userInstance = new User();

		// Set the username
		$userInstance->setEmail($email);

		// Return the instance
		return $userInstance;
	}

	/**
	 * "Getter" for databse entry
	 *
	 * @return	$entry	An array with database entries
	 * @throws	NullPointerException	If the database result is not found
	 * @throws	InvalidDatabaseResultException	If the database result is invalid
	 */
	private function getDatabaseEntry () {
		// Is there an instance?
		if (is_null($this->resultInstance)) {
			// Throw new exception
			throw new NullPointerException($this, self::EXCEPTION_IS_NULL_POINTER);
		} // END - if

		// Rewind it
		$this->resultInstance->rewind();

		// Do we have an entry?
		if (!$this->resultInstance->valid()) {
			throw new InvalidDatabaseResultException(array($this, $this->resultInstance), DatabaseResult::EXCEPTION_INVALID_DATABASE_RESULT);
		} // END - if

		// Get next entry
		$this->resultInstance->next();

		// Fetch it
		$entry = $this->resultInstance->current();

		// And return it
		return $entry;
	}

	/**
	 * Setter for username
	 *
	 * @param	$userName	The username to set
	 * @return	void
	 */
	public final function setUserName ($userName) {
		$this->userName = $userName;
	}

	/**
	 * Setter for email
	 *
	 * @param	$email	The email to set
	 * @return	void
	 */
	protected final function setEmail ($email) {
		$this->email = $email;
	}

	/**
	 * Getter for username
	 *
	 * @return	$userName	The username to get
	 */
	public final function getUsername () {
		return $this->userName;
	}

	/**
	 * Getter for email
	 *
	 * @return	$email	The email to get
	 */
	public final function getEmail () {
		return $this->email;
	}

	/**
	 * Determines wether the username exists or not
	 *
	 * @return	$exists		Wether the username exists
	 */
	public function ifUsernameExists () {
		// By default the username does not exist
		$exists = false;

		// Is a previous result there?
		if (is_null($this->resultInstance)) {
			// Get a UserDatabaseWrapper instance
			$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper_class');

			// Create a search criteria
			$criteriaInstance = ObjectFactory::createObjectByConfiguredName('search_criteria_class');

			// Add the username as a criteria and set limit to one entry
			$criteriaInstance->addCriteria(UserDatabaseWrapper::DB_COLUMN_USERNAME, $this->getUsername());
			$criteriaInstance->setLimit(1);

			// Get a search result
			$this->resultInstance = $wrapperInstance->doSelectByCriteria($criteriaInstance);
		} else {
			// Rewind it
			$this->resultInstance->rewind();
		}

		// Search for it
		if ($this->resultInstance->next()) {
			// Entry found
			$exists = true;
		} // END - if

		// Return the status
		return $exists;
	}

	/**
	 * Determines wether the email exists or not
	 *
	 * @return	$exists		Wether the email exists
	 */
	public function ifEmailAddressExists () {
		// By default the email does not exist
		$exists = false;

		// Is a previous result there?
		if (is_null($this->resultInstance)) {
			// Get a UserDatabaseWrapper instance
			$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper_class');

			// Create a search criteria
			$criteriaInstance = ObjectFactory::createObjectByConfiguredName('search_criteria_class');

			// Add the username as a criteria and set limit to one entry
			$criteriaInstance->addCriteria(UserDatabaseWrapper::DB_COLUMN_EMAIL, $this->getEmail());
			$criteriaInstance->setLimit(1);

			// Get a search resultInstance
			$this->resultInstance = $wrapperInstance->doSelectByCriteria($criteriaInstance);
		} else {
			// Rewind it
			$this->resultInstance->rewind();
		}

		// Search for it
		if ($this->resultInstance->next()) {
			// Entry found
			$exists = true;
		} // END - if

		// Return the status
		return $exists;
	}

	/**
	 * Checks if the supplied password hash in request matches with the stored
	 * in database.
	 *
	 * @param	$requestInstance	A requestable class instance
	 * @return	$matches			Wether the supplied password hash matches
	 */
	public function ifPasswordHashMatches (Requestable $requestInstance) {
		// By default nothing matches... ;)
		$matches = false;

		// Get a UserDatabaseWrapper instance
		$wrapperInstance = ObjectFactory::createObjectByConfiguredName('user_db_wrapper_class');

		// Create a search criteria
		$criteriaInstance = ObjectFactory::createObjectByConfiguredName('search_criteria_class');

		// Add the username as a criteria and set limit to one entry
		$criteriaInstance->addCriteria(UserDatabaseWrapper::DB_COLUMN_USERNAME, $this->getUserName());
		$criteriaInstance->setLimit(1);

		// Get a search resultInstance
		$this->resultInstance = $wrapperInstance->doSelectByCriteria($criteriaInstance);

		// Search for it
		if ($this->resultInstance->next()) {
			// Get the current entry (can only be one!)
			$entry = $this->resultInstance->current();

			// So does the hashes match?
			//* DEBUG: */ echo $requestInstance->getRequestElement('pass_hash')."/".$entry['pass_hash'];
			$matches = ($requestInstance->getRequestElement('pass_hash') === $entry['pass_hash']);
		} // END - if

		// Return the status
		return $matches;
	}

	/**
	 * Adds data for later complete update
	 *
	 * @param	$column		Column we want to update
	 * @param	$value		New value to store in database
	 * @return	void
	 * @todo	0% done
	 */
	public function addUpdateData ($column, $value) {
		$this->partialStub("Column={$column}, value={$value}");
	}

	/**
	 * "Getter" for user's password hash
	 *
	 * @return	$passHash	User's password hash from database result
	 */
	public function getPasswordHash () {
		// Default is missing password hash
		$passHash = null;

		// Get a database entry
		$entry = $this->getDatabaseEntry();

		// Is the password hash there?
		if (isset($entry['pass_hash'])) {
			// Get it
			$passHash = $entry['pass_hash'];
		}

		// And return the hash
		return $passHash;
	}

	/**
	 * Updates the last activity timestamp and last performed action in the
	 * database result. You should call flushUpdates() to flush these updates
	 * to the database layer.
	 *
	 * @param	$requestInstance	A requestable class instance
	 * @return	void
	 */
	public function updateLastActivity (Requestable $requestInstance) {
		// No activity will be logged for guest accounts
	}

	/**
	 * Flushs all updated entries to the database layer
	 *
	 * @return	void
	 */
	public function flushUpdates () {
		// No updates will be flushed to database!
	}
}

// [EOF]
?>
